
	/********************************************************************/
	/*                                                                  */
	/*      Copyright (C) 2004 Arash Dejkam, All Rights Reserved.       */
	/*      http://www.tracewatch.com                                   */
	/*                                                                  */
	/*      Please read the licence file distributed with this          */
	/*      file or the one available at tracewatch.com for             */
	/*      the terms under which you can use or modify this file.      */
	/*                                                                  */
	/********************************************************************/
    
function WebsiteList( parentWebsite ) {
	this.ArdeComponent( 'div' );
	if( parentWebsite ) this.cls( 'indent_pad' );
	this.parentWebsite = parentWebsite;
	this.newWebsite = new NewWebsite( this );
	this.append( this.newWebsite );
	this.websites = {};
}

WebsiteList.prototype.addWebsite = function( website ) {
	
	if( this.parentWebsite == null && website.parentId ) {
		return this.websites[ website.parentId ].websiteList.addWebsite( website );
	}
	this.websites[ website.id ] = website;
	website.parentWebsiteList = this;
	if( this.newWebsite.nextSibling() ) {
		this.insertBefore( website, this.newWebsite.nextSibling() );
	} else {
		this.append( website );
	}
};

WebsiteList.prototype.removeWebsite = function( website ) {
	this.websites[ website.id ] = null;
	this.removeChild( website );
};

ArdeClass.extend( WebsiteList, ArdeComponent );

function NewWebsite( websiteList ) {
	
	this.websiteList = websiteList;
	
	if( this.websiteList.parentWebsite == null )
		this.parentId = 0;
	else
		this.parentId = this.websiteList.parentWebsite.id;
	
	this.ArdeComponent( 'div' );
	this.cls( 'special_block' );

	websiteTemplate( this, new ArdeNodeHolder( this.element ) );
	
	var self = this;
	this.addWebsiteButton.onclick = function() { self.addWebsiteClicked(); };
	this.addWebsiteButton.afterResultReceived = function( result ) { self.addWebsiteConfirmed( result ); };
}
NewWebsite.prototype.addWebsiteClicked = function() {
	this.addWebsiteButton.request( twatchFullUrl( 'rec/rec_websites.php' ), 'a=add&'+this.getParams(), Website );
};

NewWebsite.prototype.addWebsiteConfirmed = function( website ) {
	this.websiteList.addWebsite( website );
	this.clear();
	if( this.websiteList.parentWebsite != null ) {
		this.setDisplay( false );
	}
};

NewWebsite.prototype.clear = function() {
	this.nameInput.value = '';
	this.handleInput.value = '';
	this.domainInput.value = '';

	while( this.domainsSelect.length ) {
		this.domainsSelect.options[0] = null;
	}
};

NewWebsite.prototype.getParams = function() {
	return getWebsiteCommonParams( this );
};


ArdeClass.extend( NewWebsite, ArdeComponent );


function Website( id, name, handle, parentId, domains ) {
	this.id = id;
	this.name = name;
	this.handle = handle;
	this.parentId = parentId;
	this.domains = domains;
	
	this.ArdeComponent( 'div' );
	
	websiteDiv = ardeE( 'div' ).cls( 'block' ).appendTo( this );

	websiteTemplate( this, websiteDiv );

	this.moreButton = new ArdeButton( 'More Info' ).cls( 'passive' ).setFloat( 'right' ).style( 'marginTop', '5px' );
	
	this.moreCloseButton = new ArdeButton( 'Less Info' ).cls( 'passive' ).setFloat( 'right' ).setDisplay( false ).style( 'marginTop', '5px' );


	var p = ardeE( 'p' ).appendTo( websiteDiv );
	p.append( this.moreButton ).append( this.moreCloseButton );
	p.append( new ArdeImg( baseUrl+'img/dummy.gif', '', 16, 20 ) );
	this.morePane = new ArdeComponent( 'div' ).cls( 'group' ).setDisplay( false ).appendTo( websiteDiv );
	this.morePane.append( ardeE( 'p' ).append( ardeT( 'Internal ID: ' ) ).append( ardeE( 'span' ).cls( 'fixed' ).append( ardeT( this.id ) ) ) );

	var self = this;
	
	this.moreButton.element.onclick = function () {
		self.morePane.setDisplay( true );
		self.moreButton.setDisplay( false );
		self.moreCloseButton.setDisplay( true );
	};
	this.moreCloseButton.element.onclick = function () {
		self.morePane.setDisplay( false );
		self.moreCloseButton.setDisplay( false );
		self.moreButton.setDisplay( true );
		
	};
	
	if( !parentId ) {
		this.websiteList = new WebsiteList( this );
		this.websiteList.newWebsite.setDisplay( false );
		
		this.subWebsiteButton.onclick = function() {
			self.websiteList.newWebsite.switchDisplay( false );
			return false;
		};
		this.append( this.websiteList );
	}
	
	this.applyButton.onclick = function() { self.applyClicked(); };
	this.applyButton.afterResultReceived = function( result ) { self.applyConfirmed(); };
	
	if( this.id != 1 ) {
		this.deleteButton.onclick = function() { self.deleteClicked(); };
		this.deleteButton.resultReceived = function( result ) { self.deleteConfirmed(); };
	}
	
}

Website.prototype.applyClicked = function() {
	this.applyButton.request( twatchFullUrl( 'rec/rec_websites.php' ), 'a=change&'+this.getParams(), null );
};

Website.prototype.applyConfirmed = function() {
	this.name = this.nameInput.value;
	this.handle = this.handleInput.value;
	this.domains = new Array();
	for( var i = 0; i < this.domainsSelect.options.length; ++i ) {
		this.domains.push( this.domainsSelect.options[ i ].value );
	}
};

Website.prototype.deleteClicked = function() {
	this.deleteButton.request( twatchFullUrl( 'rec/rec_websites.php' ), 'a=delete&i='+this.id );
};

Website.prototype.deleteConfirmed = function() {

	this.parentWebsiteList.removeWebsite( this );

};

Website.fromXml = function( element ) {
	var id = ArdeXml.intAttribute( element, 'id' );
	var name = ArdeXml.strElement( element, 'name' );
	var handle = ArdeXml.attribute( element, 'handle' );
	var parentId = ArdeXml.intAttribute( element, 'parent_id' );
	var domainsE = ArdeXml.element( element, 'domains' );
	var domainEs = new ArdeXmlElemIter( domainsE, 'domain' );
	var domains = new Array();
	while( domainEs.current ) {
		domains.push( ArdeXml.strContent( domainEs.current ) );
		domainEs.next();
	}
	return new Website( id, name, handle, parentId, domains );
};

Website.prototype.getParams = function() {
	var s = 'i='+this.id; 
	s += '&'+getWebsiteCommonParams( this );
	return s;
};

function getWebsiteCommonParams( component ) {
	s = '&n='+ardeEscape( component.nameInput.value );
	s += '&h='+ardeEscape( component.handleInput.value );
	s += '&p='+component.parentId;
	s += '&d=';
	for( var i = 0; i < component.domainsSelect.options.length; ++i ) {
		s += ardeEscape( (i?'|':'')+component.domainsSelect.options[ i ].value );
	}
	return s;
};

ArdeClass.extend( Website, ArdeComponent );

function websiteTemplate( component, node ) {
	if( component instanceof Website ) {
		if( component.parentId ) {
			var txt = ardeT( component.name );
		} else {
			var txt = ardeT( component.name );
		}
	} else {
		if( component.websiteList.parentWebsite ) {
			var txt = ardeT( 'New Sub-Website' );
		} else {
			var txt = ardeT( 'New Website' );
		}
	}
	node.append( ardeE( 'p' ).style( 'fontWeight', 'bold' ).append( txt ) );

	var tb = ardeE( 'tbody' ).appendTo( ardeE( 'table' ).attr( 'cellpadding', '0' ).attr( 'cellspacing', '0' ).attr( 'border', '0' ).appendTo( node ) );	
	var topTr = ardeE( 'tr' ).appendTo( tb );
	var td = ardeE( 'td' ).style( 'verticalAlign', 'top' ).appendTo( topTr );
	
	tb = ardeE( 'tbody' ).appendTo( ardeE( 'table' ).cls( 'form' ).attr( 'cellpadding', '0' ).attr( 'cellspacing', '0' ).attr( 'border', '0' ).appendTo( td ) );
	
	tr = ardeE( 'tr' ).appendTo( tb );
	ardeE( 'td' ).cls( 'head' ).append( ardeT( 'Name: ' ) ).appendTo( tr );
	component.nameInput = ardeE( 'input' ).n;
	if( component instanceof Website ) {
		component.nameInput.value = component.name;
	}
	ardeE( 'td' ).cls( 'tail' ).append( component.nameInput ).appendTo( tr );
	
	tr = ardeE( 'tr' ).appendTo( tb );
	ardeE( 'td' ).cls( 'head' ).append( ardeT( 'ID: ' ) ).appendTo( tr );
	component.handleInput = ardeE( 'input' ).n;
	if( component instanceof Website ) {
		component.handleInput.value = component.handle;
	}
	ardeE( 'td' ).cls( 'tail' ).append( component.handleInput ).appendTo( tr );

	
	if( component instanceof Website && !component.parentId ) {
		component.subWebsiteButton = document.createElement( 'input' );
		component.subWebsiteButton.className = 'passive';
		component.subWebsiteButton.type = 'button';
		component.subWebsiteButton.value = 'New Sub Website';
		ardeE( 'tr' ).append( ardeE( 'td' ).attr( 'colspan', '2' ).append( ardeE( 'p' ).append( ardeE( 'br' ) ).append( component.subWebsiteButton ) ) ).appendTo( tb );
	}
	td = ardeE( 'td' ).appendTo( topTr );
	tb = ardeE( 'tbody' ).appendTo( ardeE( 'table' ).cls( 'form' ).attr( 'cellpadding', '0' ).attr( 'cellspacing', '0' ).attr( 'border', '0' ).appendTo( td ) );
	tr = ardeE( 'tr' ).appendTo( tb );
	
	
	ardeE( 'td' ).cls( 'head' ).style( 'verticalAlign', 'top' ).append( ardeT( 'domains: ' ) ).appendTo( tr );
	
	
	component.domainInput = ardeE( 'input' ).n;
	component.domainInput.size = 30;
	
	component.domainAddButton = document.createElement( 'input' );
	component.domainAddButton.type = 'button';
	component.domainAddButton.value = 'ADD';
	component.domainAddButton.className = 'passive';
	
	component.domainsSelect = document.createElement( 'select' );
	component.domainsSelect.size = 4;
	component.domainsSelect.style.width = 270+'px';
	
	if( component instanceof Website ) {
		for( var i in component.domains ) {		
			var option = ardeE( 'option' ).append( ardeT( component.domains[i] ) ).n;
			option.value = component.domains[i];
			component.domainsSelect.appendChild( option );
		}
	}

	component.domainDeleteButton = document.createElement( 'input' );
	component.domainDeleteButton.type = 'button';
	component.domainDeleteButton.value = 'Delete Selected';
	component.domainDeleteButton.className = 'passive';
	
	component.domainAddButton.onclick = function() {
		if( component.domainInput.value != '' ) {
			var option = ardeE( 'option' ).append( ardeT( component.domainInput.value ) ).n;
			option.value = component.domainInput.value;
			component.domainsSelect.appendChild( option );
			component.domainInput.value = '';
		}
	};
	
	component.domainDeleteButton.onclick = function() {
		if( component.domainsSelect.selectedIndex >= 0 ) {
			component.domainsSelect.options[ component.domainsSelect.selectedIndex ] = null;
		}
	};
	
	var td = ardeE( 'td' ).cls( 'tail' );
	td.append( component.domainInput ).append( component.domainAddButton );
	td.append( ardeE( 'br' ) ).append( component.domainsSelect );
	td.append( ardeE( 'br' ) ).append( component.domainDeleteButton );
	td.appendTo( tr );
	
	component.buttonsP = ardeE( 'p' ).appendTo( ardeE( 'td' ).style( 'verticalAlign', 'top' ).style( 'width', '200px' ).style( 'textAlign', 'right' ).appendTo( topTr ) );
	
	if( component instanceof Website ) {
		component.applyButton = new ArdeRequestButton( 'Apply Changes' );
		component.applyButton.button.style[ 'width' ] = '150px';

		component.buttonsP.append( component.applyButton ).append( ardeE( 'br' ) ).append( ardeE( 'br' ) ).append( ardeE( 'br' ) );
		
		if( component.id != 1 ) {
			component.deleteButton = new ArdeRequestButton( 'Delete Website', "Are you sure? this will also delete all tables and data related to website '"+component.name+"' and all of it's sub-websites from database." );
			component.deleteButton.button.className = 'critical';
			component.deleteButton.button.style[ 'width' ] = '150px';
			component.buttonsP.append( component.deleteButton );
		}
		
		
	} else {
		component.addWebsiteButton = new ArdeRequestButton( 'Add Website' );
		component.addWebsiteButton.button.style[ 'width' ] = '150px';
		
		component.buttonsP.append( component.addWebsiteButton );
	}
}
