<?php
	/********************************************************************/
	/*                                                                  */
	/*      Copyright (C) 2004 Arash Dejkam, All Rights Reserved.       */
	/*      http://www.tracewatch.com                                   */
	/*                                                                  */
	/*      Please read the licence file distributed with this          */
	/*      file or the one available at tracewatch.com for             */
	/*      the terms under which you can use or modify this file.      */
	/*                                                                  */
	/********************************************************************/
    
	require_once $ardeBase->path( 'db/Db.php' );

	class ArdeDbUser {
		public $id;
		public $username;
		public $random;
		public $randomExpires;
		
		public function __construct( $id, $username, $random, $randomExpires ) {
			$this->id = $id;
			$this->username = $username;
			$this->random = $random;
			$this->randomExpires = $randomExpires;
		}
	}
	
	class ArdeDbUsers {
		
		private $db;
		
		public function __construct( ArdeDb $db ) {
			$this->db = $db;
		}
		
		public function install( $firstRandom, $overwrite = true ) {
			$this->db->createTable( '', 'u', '( id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY, rnd CHAR(32) NOT NULL, exp INT UNSIGNED NOT NULL )', $overwrite );
			$this->db->createTable( '', 'lr', '( id TINYINT UNSIGNED NOT NULL PRIMARY KEY, rnd CHAR(32) NOT NULL )', $overwrite );
			$this->db->query( 'INSERT INTO', 'lr', "VALUES( 0, '".$this->db->escape( $firstRandom )."' )" );
		}
		
		public function uninstall() {
			$this->db->dropTableIfExists( '', 'u' );
			$this->db->dropTableIfExists( '', 'lr' );
			$this->uninstallRootSession();
		}
		
		public function addUser( $id ) {
			$this->db->query( 'INSERT INTO', 'u( id, rnd, exp )', "VALUES( ".$id.", '', 0 )" );
		}
		
		public function getUserByRandom( $random ) {
			$res = $this->db->query( "SELECT id, rnd, exp - UNIX_TIMESTAMP() FROM", 'u', "WHERE rnd = '".$this->db->escape( $random )."' AND UNIX_TIMESTAMP() < exp" );
			if( !( $r = $this->db->fetchRow( $res ) ) ) return null;
			return new ArdeDbUser( (int)$r[0], '', $r[1], (int)$r[2] );
		}
		
		public function getUserById( $id ) {
			$res =	$this->db->query( "SELECT id, if( UNIX_TIMESTAMP() < exp, rnd, '' ), exp - UNIX_TIMESTAMP() FROM", 'u', "WHERE id = ".$id );
			if( !( $r = $this->db->fetchRow( $res ) ) ) return null;
			if( $r[1] == '' ) {
				$r[1] = null;
				$r[2] = 0;
			}
			return new ArdeDbUser( (int)$r[0], '', $r[1], (int)$r[2] );
		}
		
		public function setRandom( $userId, $random, $expires ) {
			if( $expires ) {
				$expires = 'UNIX_TIMESTAMP()+'.$expires;
			} else {
				$expires = '0';
			}
			$this->db->query( 'UPDATE', 'u', "SET rnd = '".$this->db->escape( $random )."', exp = ".$expires." WHERE id = ".$userId );
		}
		
		public function setLastRandom( $rnd ) {
			$this->db->query( 'UPDATE', 'lr', "SET rnd = '".$this->db->escape( $rnd )."'" );
		}
		
		public function getLastRandom( $soft = false ) {
			try {
				$res = $this->db->query( 'SELECT rnd FROM', 'lr' );
				$r = $this->db->fetchRow( $res );
				return $r[0];
			} catch( ArdeDbQueryError $e ) {
				if( $soft && $e->getCode() == $this->db->getErrorCode( ArdeDb::ERROR_TABLE_NOT_EXIST ) ) {
					return '';
				} else {
					throw $e;
				}
			}
		}
		
		public function getRootSessionRandom() {
			try {
				$res = $this->db->query( 'SELECT rnd FROM', 'rs', "WHERE id = 0 AND UNIX_TIMESTAMP() < exp" );
			} catch( ArdeDbQueryError $e ) {
				if( $e->getCode() == $this->db->getErrorCode( ArdeDb::ERROR_TABLE_NOT_EXIST ) ) return null;
				throw $e;
			}
			if( !( $r = $this->db->fetchRow( $res ) ) ) return null;
			return $r[0];
		}
		
		public function setRootSessionRandom( $random, $expires ) {
			try {
				$this->_setRootSessionRandom( $random, $expires );
			} catch( ArdeDbQueryError $e ) {
				if( $e->getCode() == $this->db->getErrorCode( ArdeDb::ERROR_TABLE_NOT_EXIST ) ) {
					$this->installRootSession();
					return $this->_setRootSessionRandom( $random, $expires );
				}
				throw $e;
			}
		}
		private function _setRootSessionRandom( $random, $expires ) {
			$this->db->query( 'INSERT INTO', 'rs', "VALUES( 0, '".$this->db->escape( $random )."', UNIX_TIMESTAMP() + ".$expires." ) ON DUPLICATE KEY UPDATE rnd = VALUES( rnd ), exp = VALUES( exp )" );
		}
		
		private function installRootSession() {
			$this->db->createTable( '', 'rs', '( id TINYINT UNSIGNED NOT NULL PRIMARY KEY, rnd CHAR(32) NOT NULL, exp INT UNSIGNED NOT NULL )' );
		}
		
		public function uninstallRootSession() {
			$this->db->dropTableIfExists( '', 'rs' );
		}
	
	}
?>